@@ -13,17 +13,17 @@
 # import types so that we can reference ListType in sphinx param declarations.
 # We can't just use list, because sphinx gets confused by
 # openstack.resource.Resource.list and openstack.resource2.Resource.list
-import jsonpatch
 import types  # noqa
 import warnings
 
-from openstack.cloud import exc
-from openstack.cloud import _normalize
+import jsonpatch
+
 from openstack.cloud import _utils
+from openstack.cloud import exc
 from openstack import utils
 
 
-class BaremetalCloudMixin(_normalize.Normalizer):
+class BaremetalCloudMixin:
 
     @property
     def _baremetal_client(self):
@@ -46,8 +46,8 @@ class BaremetalCloudMixin(_normalize.Normalizer):
     def list_nics_for_machine(self, uuid):
         """Returns a list of ports present on the machine node.
 
-        :param uuid: String representing machine UUID value in
-                     order to identify the machine.
+        :param uuid: String representing machine UUID value in order to
+            identify the machine.
         :returns: A list of ports.
         """
         # TODO(dtantsur): support node names here.
@@ -80,7 +80,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         :param name_or_id: A node name or UUID that will be looked up.
 
         :returns: ``munch.Munch`` representing the node found or None if no
-                  nodes are found.
+            nodes are found.
         """
         try:
             return self._normalize_machine(self.baremetal.get_node(name_or_id))
@@ -92,8 +92,8 @@ class BaremetalCloudMixin(_normalize.Normalizer):
 
         :param mac: Port MAC address to query in order to return a node.
 
-        :returns: ``munch.Munch`` representing the node found or None
-                  if the node is not found.
+        :returns: ``munch.Munch`` representing the node found or None if the
+            node is not found.
         """
         nic = self.get_nic_by_mac(mac)
         if nic is None:
@@ -108,16 +108,16 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         metadata about the baremetal machine.
 
         :param name_or_id: String representing machine name or UUID value in
-                           order to identify the machine.
+            order to identify the machine.
 
         :param wait: Boolean value controlling if the method is to wait for
-                     the desired state to be reached or a failure to occur.
+            the desired state to be reached or a failure to occur.
 
-        :param timeout: Integer value, defautling to 3600 seconds, for the$
-                        wait state to reach completion.
+        :param timeout: Integer value, defautling to 3600 seconds, for the
+            wait state to reach completion.
 
         :returns: ``munch.Munch`` representing the current state of the machine
-                  upon exit of the method.
+            upon exit of the method.
         """
 
         return_to_available = False
@@ -125,7 +125,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         node = self.baremetal.get_node(name_or_id)
 
         # NOTE(TheJulia): If in available state, we can do this. However,
-        # we need to to move the machine back to manageable first.
+        # we need to move the machine back to manageable first.
         if node.provision_state == 'available':
             if node.instance_id:
                 raise exc.OpenStackCloudException(
@@ -175,36 +175,34 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         created are deleted, and the node is removed from Ironic.
 
         :param nics:
-           An array of MAC addresses that represent the
-           network interfaces for the node to be created.
+            An array of MAC addresses that represent the
+            network interfaces for the node to be created.
 
-           Example::
+            Example::
 
-              [
-                  {'mac': 'aa:bb:cc:dd:ee:01'},
-                  {'mac': 'aa:bb:cc:dd:ee:02'}
-              ]
+                [
+                    {'mac': 'aa:bb:cc:dd:ee:01'},
+                    {'mac': 'aa:bb:cc:dd:ee:02'}
+                ]
 
-        :param wait: Boolean value, defaulting to false, to wait for the
-                     node to reach the available state where the node can be
-                     provisioned. It must be noted, when set to false, the
-                     method will still wait for locks to clear before sending
-                     the next required command.
+        :param wait: Boolean value, defaulting to false, to wait for the node
+            to reach the available state where the node can be provisioned. It
+            must be noted, when set to false, the method will still wait for
+            locks to clear before sending the next required command.
 
-        :param timeout: Integer value, defautling to 3600 seconds, for the
-                        wait state to reach completion.
+        :param timeout: Integer value, defautling to 3600 seconds, for the wait
+            state to reach completion.
 
         :param lock_timeout: Integer value, defaulting to 600 seconds, for
-                             locks to clear.
+            locks to clear.
 
         :param kwargs: Key value pairs to be passed to the Ironic API,
-                       including uuid, name, chassis_uuid, driver_info,
-                       parameters.
+            including uuid, name, chassis_uuid, driver_info, parameters.
 
         :raises: OpenStackCloudException on operation error.
 
         :returns: Returns a ``munch.Munch`` representing the new
-                  baremetal node.
+            baremetal node.
         """
 
         msg = ("Baremetal machine node failed to be created.")
@@ -325,25 +323,27 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         else:
             return machine
 
-    def unregister_machine(self, nics, uuid, wait=False, timeout=600):
+    def unregister_machine(self, nics, uuid, wait=None, timeout=600):
         """Unregister Baremetal from Ironic
 
         Removes entries for Network Interfaces and baremetal nodes
         from an Ironic API
 
         :param nics: An array of strings that consist of MAC addresses
-                          to be removed.
+            to be removed.
         :param string uuid: The UUID of the node to be deleted.
 
-        :param wait: Boolean value, defaults to false, if to block the method
-                     upon the final step of unregistering the machine.
+        :param wait: DEPRECATED, do not use.
 
         :param timeout: Integer value, representing seconds with a default
-                        value of 600, which controls the maximum amount of
-                        time to block the method's completion on.
+            value of 600, which controls the maximum amount of time to block
+            until a lock is released on machine.
 
         :raises: OpenStackCloudException on operation failure.
         """
+        if wait is not None:
+            warnings.warn("wait argument is deprecated and has no effect",
+                          DeprecationWarning)
 
         machine = self.get_machine(uuid)
         invalid_states = ['active', 'cleaning', 'clean wait', 'clean failed']
@@ -357,47 +357,20 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         # previously concealed by exception retry logic that detected the
         # failure, and resubitted the request in python-ironicclient.
         try:
-            self.wait_for_baremetal_node_lock(machine, timeout=timeout)
+            self.baremetal.wait_for_node_reservation(machine, timeout)
         except exc.OpenStackCloudException as e:
             raise exc.OpenStackCloudException(
                 "Error unregistering node '%s': Exception occured while"
                 " waiting to be able to proceed: %s" % (machine['uuid'], e))
 
         for nic in nics:
-            port_msg = ("Error removing NIC {nic} from baremetal API for "
-                        "node {uuid}").format(nic=nic, uuid=uuid)
-            port_url = '/ports/detail?address={mac}'.format(mac=nic['mac'])
-            port = self._baremetal_client.get(port_url, microversion=1.6,
-                                              error_message=port_msg)
-            port_url = '/ports/{uuid}'.format(uuid=port['ports'][0]['uuid'])
-            _utils._call_client_and_retry(self._baremetal_client.delete,
-                                          port_url, retry_on=[409, 503],
-                                          error_message=port_msg)
+            try:
+                port = next(self.baremetal.ports(address=nic['mac']))
+            except StopIteration:
+                continue
+            self.baremetal.delete_port(port.id)
 
-        with _utils.shade_exceptions(
-                "Error unregistering machine {node_id} from the baremetal "
-                "API".format(node_id=uuid)):
-
-            # NOTE(TheJulia): While this should not matter microversion wise,
-            # ironic assumes all calls without an explicit microversion to be
-            # version 1.0. Ironic expects to deprecate support for older
-            # microversions in future releases, as such, we explicitly set
-            # the version to what we have been using with the client library..
-            version = "1.6"
-            msg = "Baremetal machine failed to be deleted"
-            url = '/nodes/{node_id}'.format(
-                node_id=uuid)
-            _utils._call_client_and_retry(self._baremetal_client.delete,
-                                          url, retry_on=[409, 503],
-                                          error_message=msg,
-                                          microversion=version)
-
-            if wait:
-                for count in utils.iterate_timeout(
-                        timeout,
-                        "Timeout waiting for machine to be deleted"):
-                    if not self.get_machine(uuid):
-                        break
+        self.baremetal.delete_node(uuid)
 
     def patch_machine(self, name_or_id, patch):
         """Patch Machine Information
@@ -407,27 +380,27 @@ class BaremetalCloudMixin(_normalize.Normalizer):
 
         :param string name_or_id: A machine name or UUID to be updated.
         :param patch:
-           The JSON Patch document is a list of dictonary objects
-           that comply with RFC 6902 which can be found at
-           https://tools.ietf.org/html/rfc6902.
+            The JSON Patch document is a list of dictonary objects that comply
+            with RFC 6902 which can be found at
+            https://tools.ietf.org/html/rfc6902.
 
-           Example patch construction::
+            Example patch construction::
 
-               patch=[]
-               patch.append({
-                   'op': 'remove',
-                   'path': '/instance_info'
-               })
-               patch.append({
-                   'op': 'replace',
-                   'path': '/name',
-                   'value': 'newname'
-               })
-               patch.append({
-                   'op': 'add',
-                   'path': '/driver_info/username',
-                   'value': 'administrator'
-               })
+                patch=[]
+                patch.append({
+                    'op': 'remove',
+                    'path': '/instance_info'
+                })
+                patch.append({
+                    'op': 'replace',
+                    'path': '/name',
+                    'value': 'newname'
+                })
+                patch.append({
+                    'op': 'add',
+                    'path': '/driver_info/username',
+                    'value': 'administrator'
+                })
 
         :raises: OpenStackCloudException on operation error.
 
@@ -448,9 +421,9 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         :raises: OpenStackCloudException on operation error.
 
         :returns: ``munch.Munch`` containing a machine sub-dictonary consisting
-                  of the updated data returned from the API update operation,
-                  and a list named changes which contains all of the API paths
-                  that received updates.
+            of the updated data returned from the API update operation, and a
+            list named changes which contains all of the API paths that
+            received updates.
         """
         machine = self.get_machine(name_or_id)
         if not machine:
@@ -518,10 +491,9 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         """Validate parameters of the machine.
 
         :param string name_or_id: The Name or UUID value representing the
-                                  baremetal node.
+            baremetal node.
         :param bool for_deploy: If ``True``, validate readiness for deployment,
-                                otherwise validate only the power management
-                                properties.
+            otherwise validate only the power management properties.
         :raises: :exc:`~openstack.exceptions.ValidationException`
         """
         if for_deploy:
@@ -547,27 +519,24 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         config drive to be utilized.
 
         :param string name_or_id: The Name or UUID value representing the
-                                  baremetal node.
-        :param string state: The desired provision state for the
-                             baremetal node.
+            baremetal node.
+        :param string state: The desired provision state for the baremetal
+            node.
         :param string configdrive: An optional URL or file or path
-                                   representing the configdrive. In the
-                                   case of a directory, the client API
-                                   will create a properly formatted
-                                   configuration drive file and post the
-                                   file contents to the API for
-                                   deployment.
+            representing the configdrive. In the case of a directory, the
+            client API will create a properly formatted configuration drive
+            file and post the file contents to the API for deployment.
         :param boolean wait: A boolean value, defaulted to false, to control
-                             if the method will wait for the desire end state
-                             to be reached before returning.
+            if the method will wait for the desire end state to be reached
+            before returning.
         :param integer timeout: Integer value, defaulting to 3600 seconds,
-                                representing the amount of time to wait for
-                                the desire end state to be reached.
+            representing the amount of time to wait for the desire end state to
+            be reached.
 
         :raises: OpenStackCloudException on operation error.
 
         :returns: ``munch.Munch`` representing the current state of the machine
-                  upon exit of the method.
+            upon exit of the method.
         """
         node = self.baremetal.set_node_provision_state(
             name_or_id, target=state, config_drive=configdrive,
@@ -584,14 +553,13 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         Sets Baremetal maintenance state and maintenance reason.
 
         :param string name_or_id: The Name or UUID value representing the
-                                  baremetal node.
-        :param boolean state: The desired state of the node.  True being in
-                              maintenance where as False means the machine
-                              is not in maintenance mode.  This value
-                              defaults to True if not explicitly set.
+            baremetal node.
+        :param boolean state: The desired state of the node. True being in
+            maintenance where as False means the machine is not in maintenance
+            mode.  This value defaults to True if not explicitly set.
         :param string reason: An optional freeform string that is supplied to
-                              the baremetal API to allow for notation as to why
-                              the node is in maintenance state.
+            the baremetal API to allow for notation as to why the node is in
+            maintenance state.
 
         :raises: OpenStackCloudException on operation error.
 
@@ -605,13 +573,13 @@ class BaremetalCloudMixin(_normalize.Normalizer):
     def remove_machine_from_maintenance(self, name_or_id):
         """Remove Baremetal Machine from Maintenance State
 
-        Similarly to set_machine_maintenance_state, this method
-        removes a machine from maintenance state.  It must be noted
-        that this method simpily calls set_machine_maintenace_state
-        for the name_or_id requested and sets the state to False.
+        Similarly to set_machine_maintenance_state, this method removes a
+        machine from maintenance state.  It must be noted that this method
+        simpily calls set_machine_maintenace_state for the name_or_id requested
+        and sets the state to False.
 
         :param string name_or_id: The Name or UUID value representing the
-                                  baremetal node.
+            baremetal node.
 
         :raises: OpenStackCloudException on operation error.
 
@@ -625,8 +593,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         This is a method that sets the node power state to "on".
 
         :params string name_or_id: A string representing the baremetal
-                                   node to have power turned to an "on"
-                                   state.
+            node to have power turned to an "on" state.
 
         :raises: OpenStackCloudException on operation error.
 
@@ -640,8 +607,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         This is a method that sets the node power state to "off".
 
         :params string name_or_id: A string representing the baremetal
-                                   node to have power turned to an "off"
-                                   state.
+            node to have power turned to an "off" state.
 
         :raises: OpenStackCloudException on operation error.
 
@@ -657,8 +623,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         to "on".
 
         :params string name_or_id: A string representing the baremetal
-                                   node to have power turned to an "off"
-                                   state.
+            node to have power turned to an "off" state.
 
         :raises: OpenStackCloudException on operation error.
 
