@@ -15,9 +15,8 @@
 # openstack.resource.Resource.list and openstack.resource2.Resource.list
 import types  # noqa
 
-from openstack.cloud import exc
-from openstack.cloud import _normalize
 from openstack.cloud import _utils
+from openstack.cloud import exc
 from openstack import utils
 
 
@@ -29,7 +28,7 @@ def _no_pending_images(images):
     return True
 
 
-class ImageCloudMixin(_normalize.Normalizer):
+class ImageCloudMixin:
 
     def __init__(self):
         self.image_api_use_tasks = self.config.config['image_api_use_tasks']
@@ -81,7 +80,7 @@ class ImageCloudMixin(_normalize.Normalizer):
                 images.append(image)
             elif image.status.lower() != 'deleted':
                 images.append(image)
-        return self._normalize_images(images)
+        return images
 
     def get_image(self, name_or_id, filters=None):
         """Get an image by name or ID.
@@ -112,12 +111,10 @@ class ImageCloudMixin(_normalize.Normalizer):
         """ Get a image by ID
 
         :param id: ID of the image.
-        :returns: An image ``munch.Munch``.
+        :returns: An image
+            :class:`openstack.image.v2.image.Image` object.
         """
-        image = self._normalize_image(
-            self.image.get_image(image={'id': id}))
-
-        return image
+        return self.image.get_image(image={'id': id})
 
     def download_image(
             self, name_or_id, output_path=None, output_file=None,
@@ -213,10 +210,10 @@ class ImageCloudMixin(_normalize.Normalizer):
         # Task API means an image was uploaded to swift
         # TODO(gtema) does it make sense to move this into proxy?
         if self.image_api_use_tasks and (
-                self.image._IMAGE_OBJECT_KEY in image
-                or self.image._SHADE_IMAGE_OBJECT_KEY in image):
-            (container, objname) = image.get(
-                self.image._IMAGE_OBJECT_KEY, image.get(
+                self.image._IMAGE_OBJECT_KEY in image.properties
+                or self.image._SHADE_IMAGE_OBJECT_KEY in image.properties):
+            (container, objname) = image.properties.get(
+                self.image._IMAGE_OBJECT_KEY, image.properties.get(
                     self.image._SHADE_IMAGE_OBJECT_KEY)).split('/', 1)
             self.delete_object(container=container, name=objname)
 
@@ -235,7 +232,7 @@ class ImageCloudMixin(_normalize.Normalizer):
             md5=None, sha256=None,
             disk_format=None, container_format=None,
             disable_vendor_agent=True,
-            wait=False, timeout=3600,
+            wait=False, timeout=3600, tags=None,
             allow_duplicates=False, meta=None, volume=None, **kwargs):
         """Upload an image.
 
@@ -259,6 +256,8 @@ class ImageCloudMixin(_normalize.Normalizer):
                                      (optional, defaults to the
                                      os-client-config config value for this
                                      cloud)
+        :param list tags: List of tags for this image. Each tag is a string
+                          of at most 255 chars.
         :param bool disable_vendor_agent: Whether or not to append metadata
                                           flags to the image to inform the
                                           cloud in question to not expect a
@@ -304,7 +303,7 @@ class ImageCloudMixin(_normalize.Normalizer):
                 md5=md5, sha256=sha256,
                 disk_format=disk_format, container_format=container_format,
                 disable_vendor_agent=disable_vendor_agent,
-                wait=wait, timeout=timeout,
+                wait=wait, timeout=timeout, tags=tags,
                 allow_duplicates=allow_duplicates, meta=meta, **kwargs)
 
         self._get_cache(None).invalidate()
@@ -328,57 +327,3 @@ class ImageCloudMixin(_normalize.Normalizer):
         image = image or name_or_id
         return self.image.update_image_properties(
             image=image, meta=meta, **properties)
-
-    def set_volume_quotas(self, name_or_id, **kwargs):
-        """ Set a volume quota in a project
-
-        :param name_or_id: project name or id
-        :param kwargs: key/value pairs of quota name and quota value
-
-        :raises: OpenStackCloudException if the resource to set the
-            quota does not exist.
-        """
-
-        proj = self.get_project(name_or_id)
-        if not proj:
-            raise exc.OpenStackCloudException("project does not exist")
-
-        kwargs['tenant_id'] = proj.id
-        self._volume_client.put(
-            '/os-quota-sets/{tenant_id}'.format(tenant_id=proj.id),
-            json={'quota_set': kwargs},
-            error_message="No valid quota or resource")
-
-    def get_volume_quotas(self, name_or_id):
-        """ Get volume quotas for a project
-
-        :param name_or_id: project name or id
-        :raises: OpenStackCloudException if it's not a valid project
-
-        :returns: Munch object with the quotas
-        """
-        proj = self.get_project(name_or_id)
-        if not proj:
-            raise exc.OpenStackCloudException("project does not exist")
-
-        data = self._volume_client.get(
-            '/os-quota-sets/{tenant_id}'.format(tenant_id=proj.id),
-            error_message="cinder client call failed")
-        return self._get_and_munchify('quota_set', data)
-
-    def delete_volume_quotas(self, name_or_id):
-        """ Delete volume quotas for a project
-
-        :param name_or_id: project name or id
-        :raises: OpenStackCloudException if it's not a valid project or the
-                 cinder client call failed
-
-        :returns: dict with the quotas
-        """
-        proj = self.get_project(name_or_id)
-        if not proj:
-            raise exc.OpenStackCloudException("project does not exist")
-
-        return self._volume_client.delete(
-            '/os-quota-sets/{tenant_id}'.format(tenant_id=proj.id),
-            error_message="cinder client call failed")
