@@ -17,8 +17,8 @@
 #             the sdk resource objects.
 
 import datetime
+
 import munch
-import six
 
 from openstack import resource
 
@@ -117,7 +117,7 @@ def _split_filters(obj_name='', filters=None, **kwargs):
 
 
 def _to_bool(value):
-    if isinstance(value, six.string_types):
+    if isinstance(value, str):
         if not value:
             return False
         prospective = value.lower().capitalize()
@@ -140,7 +140,7 @@ def _pop_or_get(resource, key, default, strict):
         return resource.get(key, default)
 
 
-class Normalizer(object):
+class Normalizer:
     '''Mix-in class to provide the normalization functions.
 
     This is in a separate class just for on-disk source code organization
@@ -388,6 +388,9 @@ class Normalizer(object):
         ret['description'] = group.pop('description')
         ret['properties'] = group
 
+        if self._use_neutron_secgroups():
+            ret['stateful'] = group.pop('stateful', True)
+
         # Backwards compat with Neutron
         if not self.strict_mode:
             ret['tenant_id'] = project_id
@@ -563,7 +566,10 @@ class Normalizer(object):
             ret['config_drive'] = config_drive
             ret['project_id'] = project_id
             ret['tenant_id'] = project_id
-            ret['region'] = self.config.region_name
+            # TODO(efried): This is hardcoded to 'compute' because this method
+            # should only ever be used by the compute proxy. (That said, it
+            # doesn't appear to be used at all, so can we get rid of it?)
+            ret['region'] = self.config.get_region_name('compute')
             ret['cloud'] = self.config.name
             ret['az'] = az
             for key, val in ret['properties'].items():
@@ -587,16 +593,16 @@ class Normalizer(object):
         :returns:
             A list of normalized dicts with the following attributes::
 
-            [
-              {
-                "id": "this-is-a-floating-ip-id",
-                "fixed_ip_address": "192.0.2.10",
-                "floating_ip_address": "198.51.100.10",
-                "network": "this-is-a-net-or-pool-id",
-                "attached": True,
-                "status": "ACTIVE"
-              }, ...
-            ]
+                [
+                    {
+                        "id": "this-is-a-floating-ip-id",
+                        "fixed_ip_address": "192.0.2.10",
+                        "floating_ip_address": "198.51.100.10",
+                        "network": "this-is-a-net-or-pool-id",
+                        "attached": True,
+                        "status": "ACTIVE"
+                    }, ...
+                ]
 
         """
         return [
@@ -604,10 +610,19 @@ class Normalizer(object):
         ]
 
     def _normalize_floating_ip(self, ip):
-        ret = munch.Munch()
-
         # Copy incoming floating ip because of shared dicts in unittests
-        ip = ip.copy()
+        if isinstance(ip, resource.Resource):
+            ip = ip.to_dict(ignore_none=True, original_names=True)
+            location = ip.pop(
+                'location',
+                self._get_current_location(project_id=ip.get('owner')))
+        else:
+            location = self._get_current_location(
+                project_id=ip.get('owner'))
+            # This copy is to keep things from getting epically weird in tests
+            ip = ip.copy()
+
+        ret = munch.Munch(location=location)
 
         fixed_ip_address = ip.pop('fixed_ip_address', ip.pop('fixed_ip', None))
         floating_ip_address = ip.pop('floating_ip_address', ip.pop('ip', None))
@@ -979,6 +994,9 @@ class Normalizer(object):
             location=self._get_current_location(),
         )
 
+        if not self.strict_mode:
+            ret['uuid'] = c_id
+
         for key in (
                 'status',
                 'cluster_template_id',
@@ -988,7 +1006,8 @@ class Normalizer(object):
                 'create_timeout',
                 'node_count',
                 'name'):
-            ret[key] = coe_cluster.pop(key)
+            if key in coe_cluster:
+                ret[key] = coe_cluster.pop(key)
 
         ret['properties'] = coe_cluster
         return ret
@@ -1165,13 +1184,14 @@ class Normalizer(object):
 
     def _normalize_machine(self, machine):
         """Normalize Ironic Machine"""
-        machine = machine.copy()
+        if isinstance(machine, resource.Resource):
+            machine = machine._to_munch()
+        else:
+            machine = machine.copy()
 
         # Discard noise
         self._remove_novaclient_artifacts(machine)
 
-        # TODO(mordred) Normalize this resource
-
         return machine
 
     def _normalize_roles(self, roles):
