@@ -11,7 +11,7 @@
 # limitations under the License.
 import copy
 import functools
-import six
+import queue
 # import types so that we can reference ListType in sphinx param declarations.
 # We can't just use list, because sphinx gets confused by
 # openstack.resource.Resource.list and openstack.resource2.Resource.list
@@ -19,35 +19,35 @@ import types  # noqa
 import warnings
 
 import dogpile.cache
+import keystoneauth1.exceptions
+import keystoneauth1.session
 import munch
 import requests.models
 import requestsexceptions
 
-import keystoneauth1.exceptions
-import keystoneauth1.session
-
 from openstack import _log
-from openstack.cloud import exc
 from openstack.cloud import _floating_ip
 from openstack.cloud import _object_store
-from openstack.cloud import meta
 from openstack.cloud import _utils
+from openstack.cloud import exc
+from openstack.cloud import meta
 import openstack.config
 from openstack.config import cloud_region as cloud_region_mod
 from openstack import proxy
+from openstack import utils
+
 
 DEFAULT_SERVER_AGE = 5
 DEFAULT_PORT_AGE = 5
 DEFAULT_FLOAT_AGE = 5
 _CONFIG_DOC_URL = _floating_ip._CONFIG_DOC_URL
-
 DEFAULT_OBJECT_SEGMENT_SIZE = _object_store.DEFAULT_OBJECT_SEGMENT_SIZE
 # This halves the current default for Swift
 DEFAULT_MAX_FILE_SIZE = _object_store.DEFAULT_MAX_FILE_SIZE
 OBJECT_CONTAINER_ACLS = _object_store.OBJECT_CONTAINER_ACLS
 
 
-class _OpenStackCloudMixin(object):
+class _OpenStackCloudMixin:
     """Represent a connection to an OpenStack Cloud.
 
     OpenStackCloud is the entry point for all cloud operations, regardless
@@ -60,9 +60,9 @@ class _OpenStackCloudMixin(object):
     :param bool strict: Only return documented attributes for each resource
                         as per the Data Model contract. (Default False)
     """
-    _OBJECT_MD5_KEY = 'x-object-meta-x-sdk-md5'
-    _OBJECT_SHA256_KEY = 'x-object-meta-x-sdk-sha256'
-    _OBJECT_AUTOCREATE_KEY = 'x-object-meta-x-sdk-autocreated'
+    _OBJECT_MD5_KEY = 'x-sdk-md5'
+    _OBJECT_SHA256_KEY = 'x-sdk-sha256'
+    _OBJECT_AUTOCREATE_KEY = 'x-sdk-autocreated'
     _OBJECT_AUTOCREATE_CONTAINER = 'images'
 
     # NOTE(shade) shade keys were x-object-meta-x-shade-md5 - we need to check
@@ -125,7 +125,7 @@ class _OpenStackCloudMixin(object):
             def _fake_invalidate(unused):
                 pass
 
-            class _FakeCache(object):
+            class _FakeCache:
                 def invalidate(self):
                     pass
 
@@ -331,7 +331,8 @@ class _OpenStackCloudMixin(object):
             name_key = '%s:%s' % (self.name, namespace)
 
         def generate_key(*args, **kwargs):
-            arg_key = ','.join(args)
+            # TODO(frickler): make handling arg keys actually work
+            arg_key = ''
             kw_keys = sorted(kwargs.keys())
             kwargs_key = ','.join(
                 ['%s:%s' % (k, kwargs[k]) for k in kw_keys if k != 'cache'])
@@ -349,7 +350,7 @@ class _OpenStackCloudMixin(object):
     def _get_major_version_id(self, version):
         if isinstance(version, int):
             return version
-        elif isinstance(version, six.string_types + (tuple,)):
+        elif isinstance(version, (str, tuple)):
             return int(version[0])
         return version
 
@@ -712,9 +713,11 @@ class _OpenStackCloudMixin(object):
     def get_name(self):
         return self.name
 
-    def get_session_endpoint(self, service_key):
+    def get_session_endpoint(self, service_key, **kwargs):
+        if not kwargs:
+            kwargs = {}
         try:
-            return self.config.get_session_endpoint(service_key)
+            return self.config.get_session_endpoint(service_key, **kwargs)
         except keystoneauth1.exceptions.catalog.EndpointNotFound as e:
             self.log.debug(
                 "Endpoint not found in %s cloud: %s", self.name, str(e))
@@ -731,7 +734,7 @@ class _OpenStackCloudMixin(object):
                     error=str(e)))
         return endpoint
 
-    def has_service(self, service_key):
+    def has_service(self, service_key, version=None):
         if not self.config.has_service(service_key):
             # TODO(mordred) add a stamp here so that we only report this once
             if not (service_key in self._disable_warnings
@@ -742,10 +745,104 @@ class _OpenStackCloudMixin(object):
                 self._disable_warnings[service_key] = True
             return False
         try:
-            endpoint = self.get_session_endpoint(service_key)
+            kwargs = dict()
+            # If a specific version was requested - try it
+            if version is not None:
+                kwargs['min_version'] = version
+                kwargs['max_version'] = version
+            endpoint = self.get_session_endpoint(service_key, **kwargs)
         except exc.OpenStackCloudException:
             return False
         if endpoint:
             return True
         else:
             return False
+
+    def project_cleanup(
+        self,
+        dry_run=True,
+        wait_timeout=120,
+        status_queue=None,
+        filters=None,
+        resource_evaluation_fn=None
+    ):
+        """Cleanup the project resources.
+
+        Cleanup all resources in all services, which provide cleanup methods.
+
+        :param bool dry_run: Cleanup or only list identified resources.
+        :param int wait_timeout: Maximum amount of time given to each service
+            to comlete the cleanup.
+        :param queue status_queue: a threading queue object used to get current
+            process status. The queue contain processed resources.
+        :param dict filters: Additional filters for the cleanup (only resources
+            matching all filters will be deleted, if there are no other
+            dependencies).
+        :param resource_evaluation_fn: A callback function, which will be
+            invoked for each resurce and must return True/False depending on
+            whether resource need to be deleted or not.
+        """
+        dependencies = {}
+        get_dep_fn_name = '_get_cleanup_dependencies'
+        cleanup_fn_name = '_service_cleanup'
+        if not status_queue:
+            status_queue = queue.Queue()
+        for service in self.config.get_enabled_services():
+            if hasattr(self, service):
+                proxy = getattr(self, service)
+                if (
+                    proxy
+                    and hasattr(proxy, get_dep_fn_name)
+                    and hasattr(proxy, cleanup_fn_name)
+                ):
+                    deps = getattr(proxy, get_dep_fn_name)()
+                    if deps:
+                        dependencies.update(deps)
+        dep_graph = utils.TinyDAG()
+        for k, v in dependencies.items():
+            dep_graph.add_node(k)
+            for dep in v['before']:
+                dep_graph.add_node(dep)
+                dep_graph.add_edge(k, dep)
+            for dep in v.get('after', []):
+                dep_graph.add_edge(dep, k)
+
+        cleanup_resources = dict()
+
+        for service in dep_graph.walk(timeout=wait_timeout):
+            fn = None
+            if hasattr(self, service):
+                proxy = getattr(self, service)
+                cleanup_fn = getattr(proxy, cleanup_fn_name, None)
+                if cleanup_fn:
+                    fn = functools.partial(
+                        cleanup_fn,
+                        dry_run=dry_run,
+                        client_status_queue=status_queue,
+                        identified_resources=cleanup_resources,
+                        filters=filters,
+                        resource_evaluation_fn=resource_evaluation_fn
+                    )
+            if fn:
+                self._pool_executor.submit(
+                    cleanup_task, dep_graph, service, fn
+                )
+            else:
+                dep_graph.node_done(service)
+
+        for count in utils.iterate_timeout(
+                timeout=wait_timeout,
+                message="Timeout waiting for cleanup to finish",
+                wait=1):
+            if dep_graph.is_complete():
+                return
+
+
+def cleanup_task(graph, service, fn):
+    try:
+        fn()
+    except Exception:
+        log = _log.setup_logging('openstack.project_cleanup')
+        log.exception('Error in the %s cleanup function' % service)
+    finally:
+        graph.node_done(service)
