@@ -50,6 +50,12 @@ def _check_resource(strict=False):
     return wrap
 
 
+def normalize_metric_name(name):
+    name = name.replace('.', '_')
+    name = name.replace(':', '_')
+    return name
+
+
 class Proxy(adapter.Adapter):
     """Represents a service."""
 
@@ -204,22 +210,35 @@ class Proxy(adapter.Adapter):
             self._report_stats_influxdb(response, url, method, exc)
 
     def _report_stats_statsd(self, response, url=None, method=None, exc=None):
-        if response is not None and not url:
-            url = response.request.url
-        if response is not None and not method:
-            method = response.request.method
-        name_parts = self._extract_name(url,
-                                        self.service_type,
-                                        self.session.get_project_id())
-        key = '.'.join(
-            [self._statsd_prefix, self.service_type, method]
-            + name_parts)
-        if response is not None:
-            duration = int(response.elapsed.total_seconds() * 1000)
-            self._statsd_client.timing(key, duration)
-            self._statsd_client.incr(key)
-        elif exc is not None:
-            self._statsd_client.incr('%s.failed' % key)
+        try:
+            if response is not None and not url:
+                url = response.request.url
+            if response is not None and not method:
+                method = response.request.method
+            name_parts = [
+                normalize_metric_name(f) for f in
+                self._extract_name(
+                    url, self.service_type, self.session.get_project_id())
+            ]
+            key = '.'.join(
+                [self._statsd_prefix,
+                 normalize_metric_name(self.service_type), method,
+                 '_'.join(name_parts)
+                 ])
+            with self._statsd_client.pipeline() as pipe:
+                if response is not None:
+                    duration = int(response.elapsed.total_seconds() * 1000)
+                    metric_name = '%s.%s' % (key, str(response.status_code))
+                    pipe.timing(metric_name, duration)
+                    pipe.incr(metric_name)
+                    if duration > 1000:
+                        pipe.incr('%s.over_1000' % key)
+                elif exc is not None:
+                    pipe.incr('%s.failed' % key)
+                pipe.incr('%s.attempted' % key)
+        except Exception:
+            # We do not want errors in metric reporting ever break client
+            self.log.exception("Exception reporting metrics")
 
     def _report_stats_prometheus(self, response, url=None, method=None,
                                  exc=None):
@@ -253,9 +272,12 @@ class Proxy(adapter.Adapter):
             method = response.request.method
         tags = dict(
             method=method,
-            name='_'.join(self._extract_name(
-                url, self.service_type,
-                self.session.get_project_id()))
+            name='_'.join([
+                normalize_metric_name(f) for f in
+                self._extract_name(
+                    url, self.service_type,
+                    self.session.get_project_id())
+            ])
         )
         fields = dict(
             attempted=1
@@ -310,14 +332,13 @@ class Proxy(adapter.Adapter):
         """Get a resource object to work on
 
         :param resource_type: The type of resource to operate on. This should
-                              be a subclass of
-                              :class:`~openstack.resource.Resource` with a
-                              ``from_id`` method.
+            be a subclass of :class:`~openstack.resource.Resource` with a
+            ``from_id`` method.
         :param value: The ID of a resource or an object of ``resource_type``
-                      class if using an existing instance, or ``munch.Munch``,
-                      or None to create a new instance.
-        :param path_args: A dict containing arguments for forming the request
-                          URL, if needed.
+            class if using an existing instance, or ``munch.Munch``,
+            or None to create a new instance.
+        :param attrs: A dict containing arguments for forming the request
+            URL, if needed.
         """
         conn = self._get_connection()
         if value is None:
@@ -360,13 +381,13 @@ class Proxy(adapter.Adapter):
 
         :param name_or_id: The name or ID of a resource to find.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the resource does not exist.
+            When set to ``True``, None will be returned when
+            attempting to find a nonexistent resource.
         :param dict attrs: Attributes to be passed onto the
-                           :meth:`~openstack.resource.Resource.find`
-                           method, such as query parameters.
+            :meth:`~openstack.resource.Resource.find`
+            method, such as query parameters.
 
         :returns: An instance of ``resource_type`` or None
         """
@@ -374,32 +395,33 @@ class Proxy(adapter.Adapter):
                                   ignore_missing=ignore_missing,
                                   **attrs)
 
+    # TODO(stephenfin): Update docstring for attrs since it's a lie
     @_check_resource(strict=False)
     def _delete(self, resource_type, value, ignore_missing=True, **attrs):
         """Delete a resource
 
         :param resource_type: The type of resource to delete. This should
-                              be a :class:`~openstack.resource.Resource`
-                              subclass with a ``from_id`` method.
+            be a :class:`~openstack.resource.Resource`
+            subclass with a ``from_id`` method.
         :param value: The value to delete. Can be either the ID of a
-                      resource or a :class:`~openstack.resource.Resource`
-                      subclass.
+            resource or a :class:`~openstack.resource.Resource`
+            subclass.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, no exception will be set when
-                    attempting to delete a nonexistent resource.
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the resource does not exist.
+            When set to ``True``, no exception will be set when
+            attempting to delete a nonexistent resource.
         :param dict attrs: Attributes to be passed onto the
-                           :meth:`~openstack.resource.Resource.delete`
-                           method, such as the ID of a parent resource.
+            :meth:`~openstack.resource.Resource.delete`
+            method, such as the ID of a parent resource.
 
         :returns: The result of the ``delete``
         :raises: ``ValueError`` if ``value`` is a
-                 :class:`~openstack.resource.Resource` that doesn't match
-                 the ``resource_type``.
-                 :class:`~openstack.exceptions.ResourceNotFound` when
-                 ignore_missing if ``False`` and a nonexistent resource
-                 is attempted to be deleted.
+            :class:`~openstack.resource.Resource` that doesn't match
+            the ``resource_type``.
+            :class:`~openstack.exceptions.ResourceNotFound` when
+            ignore_missing if ``False`` and a nonexistent resource
+            is attempted to be deleted.
 
         """
         res = self._get_resource(resource_type, value, **attrs)
@@ -420,17 +442,17 @@ class Proxy(adapter.Adapter):
         :param resource_type: The type of resource to update.
         :type resource_type: :class:`~openstack.resource.Resource`
         :param value: The resource to update. This must either be a
-                      :class:`~openstack.resource.Resource` or an id
-                      that corresponds to a resource.
+            :class:`~openstack.resource.Resource` or an id
+            that corresponds to a resource.
         :param str base_path: Base part of the URI for updating resources, if
-                              different from
-                              :data:`~openstack.resource.Resource.base_path`.
+            different from
+            :data:`~openstack.resource.Resource.base_path`.
         :param dict attrs: Attributes to be passed onto the
-                           :meth:`~openstack.resource.Resource.update`
-                           method to be updated. These should correspond
-                           to either :class:`~openstack.resource.Body`
-                           or :class:`~openstack.resource.Header`
-                           values on this resource.
+            :meth:`~openstack.resource.Resource.update`
+            method to be updated. These should correspond
+            to either :class:`~openstack.resource.Body`
+            or :class:`~openstack.resource.Header`
+            values on this resource.
 
         :returns: The result of the ``update``
         :rtype: :class:`~openstack.resource.Resource`
@@ -444,16 +466,16 @@ class Proxy(adapter.Adapter):
         :param resource_type: The type of resource to create.
         :type resource_type: :class:`~openstack.resource.Resource`
         :param str base_path: Base part of the URI for creating resources, if
-                              different from
-                              :data:`~openstack.resource.Resource.base_path`.
+            different from
+            :data:`~openstack.resource.Resource.base_path`.
         :param path_args: A dict containing arguments for forming the request
-                          URL, if needed.
+            URL, if needed.
         :param dict attrs: Attributes to be passed onto the
-                           :meth:`~openstack.resource.Resource.create`
-                           method to be created. These should correspond
-                           to either :class:`~openstack.resource.Body`
-                           or :class:`~openstack.resource.Header`
-                           values on this resource.
+            :meth:`~openstack.resource.Resource.create`
+            method to be created. These should correspond
+            to either :class:`~openstack.resource.Body`
+            or :class:`~openstack.resource.Header`
+            values on this resource.
 
         :returns: The result of the ``create``
         :rtype: :class:`~openstack.resource.Resource`
@@ -468,14 +490,14 @@ class Proxy(adapter.Adapter):
         :param resource_type: The type of resource to create.
         :type resource_type: :class:`~openstack.resource.Resource`
         :param list data: List of attributes dicts to be passed onto the
-                          :meth:`~openstack.resource.Resource.create`
-                          method to be created. These should correspond
-                          to either :class:`~openstack.resource.Body`
-                          or :class:`~openstack.resource.Header`
-                          values on this resource.
+            :meth:`~openstack.resource.Resource.create`
+            method to be created. These should correspond
+            to either :class:`~openstack.resource.Body`
+            or :class:`~openstack.resource.Header`
+            values on this resource.
         :param str base_path: Base part of the URI for creating resources, if
-                              different from
-                              :data:`~openstack.resource.Resource.base_path`.
+            different from
+            :data:`~openstack.resource.Resource.base_path`.
 
         :returns: A generator of Resource objects.
         :rtype: :class:`~openstack.resource.Resource`
@@ -490,17 +512,17 @@ class Proxy(adapter.Adapter):
         :param resource_type: The type of resource to get.
         :type resource_type: :class:`~openstack.resource.Resource`
         :param value: The value to get. Can be either the ID of a
-                      resource or a :class:`~openstack.resource.Resource`
-                      subclass.
+            resource or a :class:`~openstack.resource.Resource`
+            subclass.
         :param str base_path: Base part of the URI for fetching resources, if
-                              different from
-                              :data:`~openstack.resource.Resource.base_path`.
+            different from
+            :data:`~openstack.resource.Resource.base_path`.
         :param dict attrs: Attributes to be passed onto the
-                           :meth:`~openstack.resource.Resource.get`
-                           method. These should correspond
-                           to either :class:`~openstack.resource.Body`
-                           or :class:`~openstack.resource.Header`
-                           values on this resource.
+            :meth:`~openstack.resource.Resource.get`
+            method. These should correspond
+            to either :class:`~openstack.resource.Body`
+            or :class:`~openstack.resource.Header`
+            values on this resource.
 
         :returns: The result of the ``fetch``
         :rtype: :class:`~openstack.resource.Resource`
@@ -512,23 +534,20 @@ class Proxy(adapter.Adapter):
             error_message="No {resource_type} found for {value}".format(
                 resource_type=resource_type.__name__, value=value))
 
-    def _list(self, resource_type, value=None,
+    def _list(self, resource_type,
               paginated=True, base_path=None, **attrs):
         """List a resource
 
         :param resource_type: The type of resource to list. This should
-                              be a :class:`~openstack.resource.Resource`
-                              subclass with a ``from_id`` method.
-        :param value: The resource to list. It can be the ID of a resource, or
-                      a :class:`~openstack.resource.Resource` object. When set
-                      to None, a new bare resource is created.
+            be a :class:`~openstack.resource.Resource`
+            subclass with a ``from_id`` method.
         :param bool paginated: When set to ``False``, expect all of the data
-                               to be returned in one response. When set to
-                               ``True``, the resource supports data being
-                               returned across multiple pages.
+            to be returned in one response. When set to
+            ``True``, the resource supports data being
+            returned across multiple pages.
         :param str base_path: Base part of the URI for listing resources, if
-                              different from
-                              :data:`~openstack.resource.Resource.base_path`.
+            different from
+            :data:`~openstack.resource.Resource.base_path`.
         :param dict attrs: Attributes to be passed onto the
             :meth:`~openstack.resource.Resource.list` method. These should
             correspond to either :class:`~openstack.resource.URI` values
@@ -536,8 +555,8 @@ class Proxy(adapter.Adapter):
 
         :returns: A generator of Resource objects.
         :raises: ``ValueError`` if ``value`` is a
-                 :class:`~openstack.resource.Resource` that doesn't match
-                 the ``resource_type``.
+            :class:`~openstack.resource.Resource` that doesn't match
+            the ``resource_type``.
         """
         return resource_type.list(
             self, paginated=paginated,
@@ -550,16 +569,16 @@ class Proxy(adapter.Adapter):
         :param resource_type: The type of resource to retrieve.
         :type resource_type: :class:`~openstack.resource.Resource`
         :param value: The value of a specific resource to retreive headers
-                      for. Can be either the ID of a resource,
-                      a :class:`~openstack.resource.Resource` subclass,
-                      or ``None``.
+            for. Can be either the ID of a resource,
+            a :class:`~openstack.resource.Resource` subclass,
+            or ``None``.
         :param str base_path: Base part of the URI for heading resources, if
-                              different from
-                              :data:`~openstack.resource.Resource.base_path`.
+            different from
+            :data:`~openstack.resource.Resource.base_path`.
         :param dict attrs: Attributes to be passed onto the
-                           :meth:`~openstack.resource.Resource.head` method.
-                           These should correspond to
-                           :class:`~openstack.resource.URI` values.
+            :meth:`~openstack.resource.Resource.head` method.
+            These should correspond to
+            :class:`~openstack.resource.URI` values.
 
         :returns: The result of the ``head`` call
         :rtype: :class:`~openstack.resource.Resource`
