@@ -11,20 +11,22 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+import io
 import operator
 import tempfile
 import uuid
 
-import six
-
-from openstack import exceptions
 from openstack.cloud import exc
 from openstack.cloud import meta
+from openstack import exceptions
+from openstack.image.v1 import image as image_v1
+from openstack.image.v2 import image
 from openstack.tests import fakes
 from openstack.tests.unit import base
 
 
-CINDER_URL = 'https://volume.example.com/v2/1c36b64c840a42cd9e9b931a369337f0'
+IMPORT_METHODS = 'glance-direct,web-download'
 
 
 class BaseTestImage(base.TestCase):
@@ -46,6 +48,16 @@ class BaseTestImage(base.TestCase):
         self.fake_search_return = {'images': [self.fake_image_dict]}
         self.container_name = self.getUniqueString('container')
 
+    def _compare_images(self, exp, real):
+        self.assertDictEqual(
+            image.Image(**exp).to_dict(computed=False),
+            real.to_dict(computed=False))
+
+    def _compare_images_v1(self, exp, real):
+        self.assertDictEqual(
+            image_v1.Image(**exp).to_dict(computed=False),
+            real.to_dict(computed=False))
+
 
 class TestImage(BaseTestImage):
 
@@ -78,7 +90,7 @@ class TestImage(BaseTestImage):
                           self.cloud.download_image, self.image_name)
 
     def test_download_image_two_outputs(self):
-        fake_fd = six.BytesIO()
+        fake_fd = io.BytesIO()
         self.assertRaises(exc.OpenStackCloudException,
                           self.cloud.download_image, self.image_name,
                           output_path='fake_path', output_file=fake_fd)
@@ -123,7 +135,7 @@ class TestImage(BaseTestImage):
 
     def test_download_image_with_fd(self):
         self._register_image_mocks()
-        output_file = six.BytesIO()
+        output_file = io.BytesIO()
         self.cloud.download_image(self.image_name, output_file=output_file)
         output_file.seek(0)
         self.assertEqual(output_file.read(), self.output)
@@ -166,9 +178,10 @@ class TestImage(BaseTestImage):
                      base_url_append='v2'),
                  json=self.fake_image_dict)
         ])
-        self.assertDictEqual(
-            self.cloud._normalize_image(self.fake_image_dict),
-            self.cloud.get_image_by_id(self.image_id))
+        self._compare_images(
+            self.fake_image_dict,
+            self.cloud.get_image_by_id(self.image_id)
+        )
         self.assert_calls()
 
     def test_get_image_id(self, cloud=None):
@@ -216,9 +229,9 @@ class TestImage(BaseTestImage):
                      'image', append=['images'], base_url_append='v2'),
                  json=self.fake_search_return)
         ])
-        self.assertEqual(
-            self.cloud._normalize_images([self.fake_image_dict]),
-            self.cloud.list_images())
+        [self._compare_images(a, b) for a, b in zip(
+            [self.fake_image_dict],
+            self.cloud.list_images())]
         self.assert_calls()
 
     def test_list_images_show_all(self):
@@ -229,9 +242,9 @@ class TestImage(BaseTestImage):
                      qs_elements=['member_status=all']),
                  json=self.fake_search_return)
         ])
-        self.assertEqual(
-            self.cloud._normalize_images([self.fake_image_dict]),
-            self.cloud.list_images(show_all=True))
+        [self._compare_images(a, b) for a, b in zip(
+            [self.fake_image_dict],
+            self.cloud.list_images(show_all=True))]
         self.assert_calls()
 
     def test_list_images_show_all_deleted(self):
@@ -244,10 +257,9 @@ class TestImage(BaseTestImage):
                      qs_elements=['member_status=all']),
                  json={'images': [self.fake_image_dict, deleted_image]})
         ])
-        self.assertEqual(
-            self.cloud._normalize_images([
-                self.fake_image_dict, deleted_image]),
-            self.cloud.list_images(show_all=True))
+        [self._compare_images(a, b) for a, b in zip(
+            [self.fake_image_dict],
+            self.cloud.list_images(show_all=True))]
         self.assert_calls()
 
     def test_list_images_no_filter_deleted(self):
@@ -259,10 +271,9 @@ class TestImage(BaseTestImage):
                      'image', append=['images'], base_url_append='v2'),
                  json={'images': [self.fake_image_dict, deleted_image]})
         ])
-        self.assertEqual(
-            self.cloud._normalize_images([
-                self.fake_image_dict, deleted_image]),
-            self.cloud.list_images(filter_deleted=False))
+        [self._compare_images(a, b) for a, b in zip(
+            [self.fake_image_dict],
+            self.cloud.list_images(filter_deleted=False))]
         self.assert_calls()
 
     def test_list_images_filter_deleted(self):
@@ -274,9 +285,9 @@ class TestImage(BaseTestImage):
                      'image', append=['images'], base_url_append='v2'),
                  json={'images': [self.fake_image_dict, deleted_image]})
         ])
-        self.assertEqual(
-            self.cloud._normalize_images([self.fake_image_dict]),
-            self.cloud.list_images())
+        [self._compare_images(a, b) for a, b in zip(
+            [self.fake_image_dict],
+            self.cloud.list_images())]
         self.assert_calls()
 
     def test_list_images_string_properties(self):
@@ -289,9 +300,10 @@ class TestImage(BaseTestImage):
                  json={'images': [image_dict]}),
         ])
         images = self.cloud.list_images()
-        self.assertEqual(
-            self.cloud._normalize_images([image_dict]),
-            images)
+        [self._compare_images(a, b) for a, b in zip(
+            [image_dict],
+            images)]
+
         self.assertEqual(
             images[0]['properties']['properties'],
             'list,of,properties')
@@ -312,19 +324,32 @@ class TestImage(BaseTestImage):
                      qs_elements=['marker={marker}'.format(marker=marker)]),
                  json=self.fake_search_return)
         ])
-        self.assertEqual(
-            self.cloud._normalize_images([
-                self.fake_image_dict, self.fake_image_dict]),
-            self.cloud.list_images())
+        [self._compare_images(a, b) for a, b in zip(
+            [self.fake_image_dict],
+            self.cloud.list_images())]
         self.assert_calls()
 
-    def test_create_image_put_v2(self):
+    def test_create_image_put_v2_no_import(self):
         self.cloud.image_api_use_tasks = False
 
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'image', append=['images'], base_url_append='v2'),
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri=self.get_mock_url(
@@ -342,7 +367,8 @@ class TestImage(BaseTestImage):
                          u'owner_specified.openstack.sha256':
                              self.fake_image_dict[
                                  'owner_specified.openstack.sha256'],
-                         u'visibility': u'private'})
+                         u'visibility': u'private',
+                         u'tags': [u'tag1', u'tag2']})
                  ),
             dict(method='PUT',
                  uri=self.get_mock_url(
@@ -358,15 +384,166 @@ class TestImage(BaseTestImage):
             dict(method='GET',
                  uri=self.get_mock_url(
                      'image', append=['images'], base_url_append='v2'),
+                 complete_qs=True,
                  json=self.fake_search_return)
         ])
 
         self.cloud.create_image(
             self.image_name, self.imagefile.name, wait=True, timeout=1,
-            is_public=False)
+            tags=['tag1', 'tag2'],
+            is_public=False, validate_checksum=True)
 
         self.assert_calls()
-        self.assertEqual(self.adapter.request_history[5].text.read(),
+        self.assertEqual(self.adapter.request_history[7].text.read(),
+                         self.output)
+
+    def test_create_image_put_v2_import_supported(self):
+        self.cloud.image_api_use_tasks = False
+
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
+                 json={'images': []}),
+            dict(method='POST',
+                 uri=self.get_mock_url(
+                     'image', append=['images'], base_url_append='v2'),
+                 json=self.fake_image_dict,
+                 headers={
+                     'OpenStack-image-import-methods': IMPORT_METHODS,
+                 },
+                 validate=dict(
+                     json={
+                         u'container_format': u'bare',
+                         u'disk_format': u'qcow2',
+                         u'name': self.image_name,
+                         u'owner_specified.openstack.md5':
+                             self.fake_image_dict[
+                                 'owner_specified.openstack.md5'],
+                         u'owner_specified.openstack.object': self.object_name,
+                         u'owner_specified.openstack.sha256':
+                             self.fake_image_dict[
+                                 'owner_specified.openstack.sha256'],
+                         u'visibility': u'private',
+                         u'tags': [u'tag1', u'tag2']})
+                 ),
+            dict(method='PUT',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_id, 'file'],
+                     base_url_append='v2'),
+                 request_headers={'Content-Type': 'application/octet-stream'}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.fake_image_dict['id']],
+                     base_url_append='v2'
+                 ),
+                 json=self.fake_image_dict),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'], base_url_append='v2'),
+                 complete_qs=True,
+                 json=self.fake_search_return)
+        ])
+
+        self.cloud.create_image(
+            self.image_name, self.imagefile.name, wait=True, timeout=1,
+            tags=['tag1', 'tag2'],
+            is_public=False, validate_checksum=True)
+
+        self.assert_calls()
+        self.assertEqual(self.adapter.request_history[7].text.read(),
+                         self.output)
+
+    def test_create_image_use_import(self):
+        self.cloud.image_api_use_tasks = False
+
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
+                 json={'images': []}),
+            dict(method='POST',
+                 uri=self.get_mock_url(
+                     'image', append=['images'], base_url_append='v2'),
+                 json=self.fake_image_dict,
+                 headers={
+                     'OpenStack-image-import-methods': IMPORT_METHODS,
+                 },
+                 validate=dict(
+                     json={
+                         u'container_format': u'bare',
+                         u'disk_format': u'qcow2',
+                         u'name': self.image_name,
+                         u'owner_specified.openstack.md5':
+                             self.fake_image_dict[
+                                 'owner_specified.openstack.md5'],
+                         u'owner_specified.openstack.object': self.object_name,
+                         u'owner_specified.openstack.sha256':
+                             self.fake_image_dict[
+                                 'owner_specified.openstack.sha256'],
+                         u'visibility': u'private',
+                         u'tags': [u'tag1', u'tag2']})
+                 ),
+            dict(method='PUT',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_id, 'stage'],
+                     base_url_append='v2'),
+                 request_headers={'Content-Type': 'application/octet-stream'}),
+            dict(method='POST',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_id, 'import'],
+                     base_url_append='v2'),
+                 json={'method': {'name': 'glance-direct'}}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.fake_image_dict['id']],
+                     base_url_append='v2'
+                 ),
+                 json=self.fake_image_dict),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'], base_url_append='v2'),
+                 complete_qs=True,
+                 json=self.fake_search_return)
+        ])
+
+        self.cloud.create_image(
+            self.image_name, self.imagefile.name, wait=True, timeout=1,
+            tags=['tag1', 'tag2'],
+            is_public=False, validate_checksum=True,
+            use_import=True,
+        )
+
+        self.assert_calls()
+        self.assertEqual(self.adapter.request_history[7].text.read(),
                          self.output)
 
     def test_create_image_task(self):
@@ -391,7 +568,21 @@ class TestImage(BaseTestImage):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'image', append=['images'], base_url_append='v2'),
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='HEAD',
                  uri='{endpoint}/{container}'.format(
@@ -435,10 +626,10 @@ class TestImage(BaseTestImage):
                      object=self.image_name),
                  status_code=201,
                  validate=dict(
-                     headers={'x-object-meta-x-sdk-md5':
+                     headers={'X-Object-Meta-x-sdk-md5':
                               self.fake_image_dict[
                                   'owner_specified.openstack.md5'],
-                              'x-object-meta-x-sdk-sha256':
+                              'X-Object-Meta-x-sdk-sha256':
                               self.fake_image_dict[
                                   'owner_specified.openstack.sha256']})
                  ),
@@ -518,23 +709,25 @@ class TestImage(BaseTestImage):
             dict(method='GET',
                  uri=self.get_mock_url(
                      'image', append=['images'], base_url_append='v2'),
+                 complete_qs=True,
                  json=self.fake_search_return)
         ])
 
         self.cloud.create_image(
             self.image_name, self.imagefile.name, wait=True, timeout=1,
             disk_format='vhd', container_format='ovf',
-            is_public=False, container=self.container_name)
+            is_public=False, validate_checksum=True,
+            container=self.container_name)
 
         self.assert_calls()
 
     def test_delete_autocreated_no_tasks(self):
-        self.use_nothing()
+        self.use_keystone_v3()
         self.cloud.image_api_use_tasks = False
         deleted = self.cloud.delete_autocreated_image_objects(
             container=self.container_name)
         self.assertFalse(deleted)
-        self.assert_calls()
+        self.assert_calls([])
 
     def test_delete_image_task(self):
         self.cloud.image_api_use_tasks = True
@@ -641,8 +834,10 @@ class TestImage(BaseTestImage):
                      'Date': 'Thu, 16 Nov 2017 15:24:30 GMT',
                      'Accept-Ranges': 'bytes',
                      'Content-Type': 'application/octet-stream',
-                     self.cloud._OBJECT_AUTOCREATE_KEY: 'true',
-                     'Etag': fakes.NO_MD5}),
+                     ('X-Object-Meta-'
+                      + self.cloud._OBJECT_AUTOCREATE_KEY): 'true',
+                     'Etag': fakes.NO_MD5,
+                     'X-Static-Large-Object': 'false'}),
             dict(method='DELETE',
                  uri='{endpoint}/{container}/{object}'.format(
                      endpoint=endpoint, container=self.container_name,
@@ -667,7 +862,7 @@ class TestImage(BaseTestImage):
         imagefile.close()
         self.cloud.create_image(
             name, imagefile.name, wait=True, timeout=1,
-            is_public=False, **kwargs)
+            is_public=False, validate_checksum=True, **kwargs)
 
     def test_create_image_put_v1(self):
         self.cloud.config.config['image_api_version'] = '1'
@@ -687,7 +882,11 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v1/images/detail',
+                 uri='https://image.example.com/v1/images/' + self.image_name,
+                 status_code=404),
+            dict(method='GET',
+                 uri='https://image.example.com/v1/images/detail?name='
+                 + self.image_name,
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v1/images',
@@ -706,7 +905,8 @@ class TestImage(BaseTestImage):
                  json={'images': [ret]}),
         ])
         self._call_create_image(self.image_name)
-        self.assertEqual(self._munch_images(ret), self.cloud.list_images())
+        [self._compare_images_v1(b, a) for a, b in zip(
+            self.cloud.list_images(), [ret])]
 
     def test_create_image_put_v1_bad_delete(self):
         self.cloud.config.config['image_api_version'] = '1'
@@ -727,7 +927,11 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v1/images/detail',
+                 uri='https://image.example.com/v1/images/' + self.image_name,
+                 status_code=404),
+            dict(method='GET',
+                 uri='https://image.example.com/v1/images/detail?name='
+                 + self.image_name,
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v1/images',
@@ -793,7 +997,22 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v2/images',
@@ -829,10 +1048,6 @@ class TestImage(BaseTestImage):
         fake_image['owner_specified.openstack.sha256'] = 'b'
 
         self.register_uris([
-            dict(method='GET',
-                 uri=self.get_mock_url(
-                     'image', append=['images'], base_url_append='v2'),
-                 json={'images': []}),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'image', append=['images'], base_url_append='v2'),
@@ -871,7 +1086,8 @@ class TestImage(BaseTestImage):
             exceptions.SDKException,
             self.cloud.create_image,
             self.image_name, self.imagefile.name,
-            is_public=False, md5='a', sha256='b'
+            is_public=False, md5='a', sha256='b',
+            allow_duplicates=True, validate_checksum=True
         )
 
         self.assert_calls()
@@ -879,15 +1095,10 @@ class TestImage(BaseTestImage):
     def test_create_image_put_bad_int(self):
         self.cloud.image_api_use_tasks = False
 
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://image.example.com/v2/images',
-                 json={'images': []}),
-        ])
-
         self.assertRaises(
             exc.OpenStackCloudException,
             self._call_create_image, self.image_name,
+            allow_duplicates=True,
             min_disk='fish', min_ram=0)
 
         self.assert_calls()
@@ -911,7 +1122,22 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v2/images',
@@ -932,6 +1158,7 @@ class TestImage(BaseTestImage):
                  json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
+                 complete_qs=True,
                  json={'images': [ret]}),
         ])
 
@@ -960,7 +1187,22 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v2/images',
@@ -981,6 +1223,7 @@ class TestImage(BaseTestImage):
                  json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
+                 complete_qs=True,
                  json={'images': [ret]}),
         ])
 
@@ -1010,7 +1253,22 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v2/images',
@@ -1031,12 +1289,13 @@ class TestImage(BaseTestImage):
                  json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
+                 complete_qs=True,
                  json={'images': [ret]}),
         ])
 
         self._call_create_image(
             self.image_name, min_disk='0', min_ram=0,
-            properties={'int_v': 12345}, protected=False)
+            properties={'int_v': 12345}, is_protected=False)
 
         self.assert_calls()
 
@@ -1045,7 +1304,9 @@ class TestImageSuburl(BaseTestImage):
 
     def setUp(self):
         super(TestImageSuburl, self).setUp()
-        self.use_keystone_v3(catalog='catalog-v3-suburl.json')
+        self.os_fixture.use_suburl()
+        self.os_fixture.build_tokens()
+        self.use_keystone_v3()
         self.use_glance(
             image_version_json='image-version-suburl.json',
             image_discovery_url='https://example.com/image')
@@ -1057,9 +1318,9 @@ class TestImageSuburl(BaseTestImage):
                      'image', append=['images'], base_url_append='v2'),
                  json=self.fake_search_return)
         ])
-        self.assertEqual(
-            self.cloud._normalize_images([self.fake_image_dict]),
-            self.cloud.list_images())
+        [self._compare_images(b, a) for a, b in zip(
+            self.cloud.list_images(),
+            [self.fake_image_dict])]
         self.assert_calls()
 
     def test_list_images_paginated(self):
@@ -1077,10 +1338,9 @@ class TestImageSuburl(BaseTestImage):
                      qs_elements=['marker={marker}'.format(marker=marker)]),
                  json=self.fake_search_return)
         ])
-        self.assertEqual(
-            self.cloud._normalize_images([
-                self.fake_image_dict, self.fake_image_dict]),
-            self.cloud.list_images())
+        [self._compare_images(b, a) for a, b in zip(
+            self.cloud.list_images(),
+            [self.fake_image_dict, self.fake_image_dict])]
         self.assert_calls()
 
 
@@ -1142,12 +1402,11 @@ class TestImageVolume(BaseTestImage):
         self.volume_id = str(uuid.uuid4())
 
     def test_create_image_volume(self):
-
         self.register_uris([
             self.get_cinder_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'volumev2', append=['volumes', self.volume_id, 'action']),
+                     'volumev3', append=['volumes', self.volume_id, 'action']),
                  json={'os-volume_upload_image': {'image_id': self.image_id}},
                  validate=dict(json={
                      u'os-volume_upload_image': {
@@ -1179,7 +1438,7 @@ class TestImageVolume(BaseTestImage):
             self.get_cinder_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'volumev2', append=['volumes', self.volume_id, 'action']),
+                     'volumev3', append=['volumes', self.volume_id, 'action']),
                  json={'os-volume_upload_image': {'image_id': self.image_id}},
                  validate=dict(json={
                      u'os-volume_upload_image': {
