@@ -9,19 +9,20 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import hashlib
+
+import io
 import operator
-import six
 import tempfile
+from unittest import mock
 
 from keystoneauth1 import adapter
-import mock
 import requests
-from openstack.tests.unit import base
 
 from openstack import _log
 from openstack import exceptions
 from openstack.image.v2 import image
+from openstack.tests.unit import base
+from openstack import utils
 
 IDENTIFIER = 'IDENTIFIER'
 EXAMPLE = {
@@ -83,19 +84,19 @@ EXAMPLE = {
     'auto_disk_config': True,
     'os_type': '49',
     'os_admin_user': 'ubuntu',
-    'hw_qemu_guest_agent': True,
+    'hw_qemu_guest_agent': 'yes',
     'os_require_quiesce': True,
 }
 
 
 def calculate_md5_checksum(data):
-    checksum = hashlib.md5()
+    checksum = utils.md5(usedforsecurity=False)
     for chunk in data:
         checksum.update(chunk)
     return checksum.hexdigest()
 
 
-class FakeResponse(object):
+class FakeResponse:
     def __init__(self, response, status_code=200, headers=None, reason=None):
         self.body = response
         self.content = response
@@ -139,25 +140,29 @@ class TestImage(base.TestCase):
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
-        self.assertDictEqual({'created_at': 'created_at',
-                              'is_hidden': 'os_hidden',
-                              'limit': 'limit',
-                              'marker': 'marker',
-                              'member_status': 'member_status',
-                              'name': 'name',
-                              'owner': 'owner',
-                              'protected': 'protected',
-                              'size_max': 'size_max',
-                              'size_min': 'size_min',
-                              'sort': 'sort',
-                              'sort_dir': 'sort_dir',
-                              'sort_key': 'sort_key',
-                              'status': 'status',
-                              'tag': 'tag',
-                              'updated_at': 'updated_at',
-                              'visibility': 'visibility'
-                              },
-                             sot._query_mapping._mapping)
+        self.assertDictEqual(
+            {
+                'created_at': 'created_at',
+                'id': 'id',
+                'is_hidden': 'os_hidden',
+                'limit': 'limit',
+                'marker': 'marker',
+                'member_status': 'member_status',
+                'name': 'name',
+                'owner': 'owner',
+                'protected': 'protected',
+                'size_max': 'size_max',
+                'size_min': 'size_min',
+                'sort': 'sort',
+                'sort_dir': 'sort_dir',
+                'sort_key': 'sort_key',
+                'status': 'status',
+                'tag': 'tag',
+                'updated_at': 'updated_at',
+                'visibility': 'visibility'
+            },
+            sot._query_mapping._mapping,
+        )
 
     def test_make_it(self):
         sot = image.Image(**EXAMPLE)
@@ -168,6 +173,7 @@ class TestImage(base.TestCase):
         self.assertEqual(EXAMPLE['disk_format'], sot.disk_format)
         self.assertEqual(EXAMPLE['min_disk'], sot.min_disk)
         self.assertEqual(EXAMPLE['name'], sot.name)
+        self.assertEqual(EXAMPLE['owner'], sot.owner)
         self.assertEqual(EXAMPLE['owner'], sot.owner_id)
         self.assertEqual(EXAMPLE['properties'], sot.properties)
         self.assertFalse(sot.is_protected)
@@ -276,9 +282,15 @@ class TestImage(base.TestCase):
             json={"method": {"name": "glance-direct"}}
         )
 
-    def test_import_image_with_stores(self):
+    def test_import_image_with_store(self):
         sot = image.Image(**EXAMPLE)
-        json = {"method": {"name": "web-download", "uri": "such-a-good-uri"}}
+        json = {
+            "method": {
+                "name": "web-download",
+                "uri": "such-a-good-uri",
+            },
+            "stores": ["ceph_1"],
+        }
         store = mock.MagicMock()
         store.id = "ceph_1"
         sot.import_image(self.sess, "web-download", "such-a-good-uri", store)
@@ -288,6 +300,50 @@ class TestImage(base.TestCase):
             json=json
         )
 
+    def test_import_image_with_stores(self):
+        sot = image.Image(**EXAMPLE)
+        json = {
+            "method": {
+                "name": "web-download",
+                "uri": "such-a-good-uri",
+            },
+            "stores": ["ceph_1"],
+        }
+        store = mock.MagicMock()
+        store.id = "ceph_1"
+        sot.import_image(
+            self.sess,
+            "web-download",
+            "such-a-good-uri",
+            stores=[store],
+        )
+        self.sess.post.assert_called_with(
+            'images/IDENTIFIER/import',
+            headers={},
+            json=json,
+        )
+
+    def test_import_image_with_all_stores(self):
+        sot = image.Image(**EXAMPLE)
+        json = {
+            "method": {
+                "name": "web-download",
+                "uri": "such-a-good-uri",
+            },
+            "all_stores": True,
+        }
+        sot.import_image(
+            self.sess,
+            "web-download",
+            "such-a-good-uri",
+            all_stores=True,
+        )
+        self.sess.post.assert_called_with(
+            'images/IDENTIFIER/import',
+            headers={},
+            json=json,
+        )
+
     def test_upload(self):
         sot = image.Image(**EXAMPLE)
 
@@ -403,7 +459,7 @@ class TestImage(base.TestCase):
         self.assertEqual(rv, resp)
 
     def test_image_download_output_fd(self):
-        output_file = six.BytesIO()
+        output_file = io.BytesIO()
         sot = image.Image(**EXAMPLE)
         response = mock.Mock()
         response.status_code = 200
